;; Copyright (C) 2024, 2025 Ajrat Makhmutov <rauty@altlinux.org>
;;
;; This file is part of alterator-kopidel.
;;
;; alterator-kopidel is free software: you can redistribute it and/or modify it under the terms
;; of the GNU General Public License as published by the Free Software Foundation,
;; either version 3 of the License, or (at your option) any later version.
;;
;; alterator-kopidel is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
;; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
;; See the GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along with alterator-kopidel.
;; If not, see <https://www.gnu.org/licenses/>.

(define default_ignored_files "/usr/share/alterator-kopidel/default-ignored-files.txt")
(define boot_mode (if (file-exists? "/sys/firmware/efi") "UEFI" "legacy"))

(document:surround "/std/frame")
(vbox align "top" margin 10
	(hbox align "left"
		(label text (_ "Working directory:"))
		(combobox name "workdir" current 1)
	)

	(separator)
	(checkbox
		text (_ "Use the default list of files to ignore when copying")
		name "chbx_default_ignored_files"
		state #t
		(when toggled
			(hbox_ignored_files activity (not (form-value "chbx_default_ignored_files")))
			(custom_ignored_files alterability (not (form-value "chbx_default_ignored_files")))
			(if (form-value "chbx_default_ignored_files") (let()
				(custom_ignored_files value default_ignored_files)
				(form-update-enum "workdir" (woo "workdirs_list" "/kopidel"))
			))
		)
	)
	(document:id hbox_ignored_files (hbox activity (not (form-value "chbx_default_ignored_files"))
		(label text (_ "Path to the custom list of ignored files:"))
		(document:id custom_ignored_files (edit
			value default_ignored_files
			alterability (not (form-value "chbx_default_ignored_files"))
			(when changed (form-update-enum "workdir" (woo
				"workdirs_list" "/kopidel"
				"ignored_files" (custom_ignored_files value)
			)))
		))
	))

	(separator)
	(checkbox
		text (_ "Installation in OEM mode")
		name "chbx_oem_mode"
		state #f
	)

	(if (string=? boot_mode "UEFI") (separator))
	(if (string=? boot_mode "UEFI")
		(hbox align "left"
			(label text (_ "Device for installing GRUB:"))
			(combobox name "grub device" current 1)
		)
	)

	(separator)
	(checkbox
		text (_ "Compress the copied file system using xz")
		name "chbx_compress_copied"
		state #f
	)

)
(document:id create_razlivochniy_obraz_btn (button
	text (_ "Create a razlivochniy obraz")
	align "center"
	(when clicked
		(create_razlivochniy_obraz_btn activity #f)
		(woo "create_razlivochniy_obraz" "kopidel"
			"workdir" (form-value "workdir")
			"async" #t
			"ignored_files" (if (form-value "chbx_default_ignored_files") "" (custom_ignored_files value))
			"grub_device" (if (string=? boot_mode "UEFI") (form-value "grub device") "root_dev")
			"oem_mode" (form-value "chbx_oem_mode")
			"compress_copied" (form-value "chbx_compress_copied")
		)
	)
))
(vbox align "bottom" margin 10
	(document:id main_progress (progressbar value 0 maximum 100 align "bottom" text (_ "Main progress bar")))
	(document:id step_progress (progressbar value 0 maximum 100 align "bottom" text (_ "Current step progress bar")))
)

(define (main-handler args) (and-let* ((step (cond-plistq 'step: args #f)))
	(main_progress text (cond
		((string=? (symbol->string step) "check_parameters") (_ "Checking the parameters"))
		((string=? (symbol->string step) "prepare_workdir") (_ "Preparing the working directory"))
		((string=? (symbol->string step) "create_copied_fs") (_ "Copying the file system"))
		((string=? (symbol->string step) "create_autoinstall_scm") (_ "Creating the autoinstall.scm"))
		((string=? (symbol->string step) "create_disk_partition_info") (_ "Generating disk partition information"))
		((string=? (symbol->string step) "create_system_iso") (_ "Creating an ISO image"))
		((string=? (symbol->string step) "done") (_ "Successfully created a razlivochniy obraz"))
		(else (string-append "Error: no such main_progress: " (symbol->string step)))
	))
	(if (string=? (symbol->string step) "done")
		(let()
			(create_razlivochniy_obraz_btn activity #t)
			(step_progress value 100 maximum 100)
		)
		(step_progress value 0 maximum 100)
	)
	(main_progress value (cond-plistq 'main_progress: args #f))
))
(add-mailbox main-handler)

(define (step-handler args) (and-let* ((progress (cond-plistq 'step_progress: args #f)))
	(if (equal? progress -1)
		(step_progress value 0 maximum 0)
		(step_progress value progress maximum 100 text "")
	)
))
(add-mailbox step-handler)

(if (string=? boot_mode "UEFI")
	(form-update-enum "grub device" (woo-list "/grub/devices"))
)
(form-update-enum "workdir" (woo "workdirs_list" "/kopidel"))
